<?php

namespace Lara;
use Lara\Widgets\GoogleAnalytics as GoogleAnalytics;
use Lara\Utils\Common            as Common;

/**
 * @package    Google Analytics by Lara
 * @author     Amr M. Ibrahim <mailamr@gmail.com>
 * @link       https://www.xtraorbit.com/
 * @copyright  Copyright (c) XtraOrbit Web development SRL 2016 - 2020
 */

if (!defined("ABSPATH"))
    die("This file cannot be accessed directly");

require_once("exception.class.php");

if (isset($lrdata['action']) && !empty($lrperm)){
	
	require("callURL.class.php");
	require_once("storage.class.php");
	require("GoogleAnalyticsAPI.class.php");
	require("lrgawidget.class.php");

	if (in_array("lrgawidget_perm_admin", $lrperm["permissions"])){
	    Common\ErrorHandler::setDebugMode(true);
	}
	
	$call = new GoogleAnalytics\lrgawidget($lrperm);
	$call->setSystemTimeZone(get_option('timezone_string'));

	$call->setDateRange(date('Y-m-d', strtotime('-1 month')), date('Y-m-d'));
 	
	switch ($lrdata['action']) {
		case "getAuthURL":
			if (in_array("lrgawidget_perm_admin", $lrperm["permissions"])){$call->getAuthURL(trim($lrdata['client_id']), trim($lrdata['client_secret']));}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}		
			break;	
		case "getAccessToken":
			if (in_array("lrgawidget_perm_admin", $lrperm["permissions"])){$call->getAccessToken(trim($lrdata['client_id']), trim($lrdata['client_secret']), trim($lrdata['code']));}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}		
			break;	
		case "getProfiles":
			if (in_array("lrgawidget_perm_admin", $lrperm["permissions"])){$call->getProfiles();}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}		
			break;
		case "setProfileID":
			if (in_array("lrgawidget_perm_admin", $lrperm["permissions"])){ $call->setProfileID($lrdata['account_id'],$lrdata['property_id'],$lrdata['profile_id'], $lrdata['profile_timezone'], @$lrdata['enable_universal_tracking'], @$lrdata['enable_ecommerce_graph']);}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}			
			break;	
		case "settingsReset":
			if (in_array("lrgawidget_perm_admin", $lrperm["permissions"])){ $call->settingsReset();}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}			
			break;			
		case "getSessions":
			if (in_array("lrgawidget_perm_sessions", $lrperm["permissions"])){$call->getSessions();}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}
			break;
		case "getPages":
			if (in_array("lrgawidget_perm_pages", $lrperm["permissions"])){$call->getPages();}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}		
			break;			
		case "getBrowsers":
			if (in_array("lrgawidget_perm_browsers", $lrperm["permissions"])){$call->getBrowsers(@$lrdata['versions']);}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}		
			break;
		case "getLanguages":
			if (in_array("lrgawidget_perm_languages", $lrperm["permissions"])){$call->getLanguages();}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}		
			break;	
		case "getOS":
			if (in_array("lrgawidget_perm_os", $lrperm["permissions"])){$call->getOS(@$lrdata['versions']);}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}		
			break;
		case "getDevices":
			if (in_array("lrgawidget_perm_devices", $lrperm["permissions"])){$call->getDevices(@$lrdata['versions']);}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}		
			break;			
		case "getScreenResolution":
			if (in_array("lrgawidget_perm_screenres", $lrperm["permissions"])){$call->getScreenResolution();}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}		
			break;
		case "getGraphData":
			if (in_array("lrgawidget_perm_graph_options", $lrperm["permissions"])){$call->getGraphData();}
			else{ Common\ErrorHandler::FatalError(__('You do not have permission to access this tab!', 'lara-google-analytics'));}
			break;
		default:
		    exit;
	} 
}else{ Common\ErrorHandler::FatalError(__('Fatal Error', 'lara-google-analytics'),__('Invalid Call', 'lara-google-analytics'));}

?>