<?php

namespace Lara\Utils\Common;

/**
 * @package    Google Analytics by Lara
 * @author     Amr M. Ibrahim <mailamr@gmail.com>
 * @link       https://www.xtraorbit.com/
 * @copyright  Copyright (c) XtraOrbit Web development SRL 2016 - 2020
 */

if (!defined("ABSPATH"))
    die("This file cannot be accessed directly");

class WPHelperFunctions {
	
	private static $initialized  = false;
	private static $user_id;
	private static $blog_options = array();
	
	public static function initInstance() {

		if (self::$initialized){
			return;
		}

		if (is_admin()){
			self::update_plugin();
		}

		self::$user_id      = self::current_user_id();
		self::$blog_options = self::blog_options();
		self::$initialized  = true;
        }

	private static function update_plugin(){
		
		$version = get_network_option(1,lrgawidget_plugin_prefiex.'version', '1.0');
		if (version_compare($version, lrgawidget_plugin_version, '<')){
			global $wpdb;
			if (version_compare($version, '3.0.0', '<')){
				$old_settings = array();
				$results = $wpdb->get_results ( "SELECT `name`, `value` FROM  `".lrgawidget_plugin_table."`", ARRAY_A );
				if (!empty($results)){
					foreach ($results as $setting) {
						$old_settings[$setting['name']] = $setting['value'];
					}			
				}

				$property_id = get_option('lrgawidget_property_id',"");
				if ( (!empty($property_id) && !self::is_analytics($property_id)) || (!empty($old_settings['property_id']) && !self::is_analytics($old_settings['property_id']))	){
					$wpdb->query("TRUNCATE TABLE `".lrgawidget_plugin_table."`");
					if (!session_id()){session_start();}
					foreach ($_SESSION as $key => $value) {
						if(preg_match('/^lrgatmp_/s', $key)){
							unset($_SESSION[$key]);
						}
					}
					$property_id = "";
					$old_settings = array();
				}
				
				if (!empty($property_id) && self::is_analytics($property_id)) {
					$wpdb->insert( lrgawidget_plugin_table, array( 'name' => 'enable_universal_tracking', 'value' => 'on'));
				}else{
					if(!empty($old_settings)){
						$wpdb->insert( lrgawidget_plugin_table, array( 'name' => 'enable_universal_tracking', 'value' => 'off'));
					}
				}
				delete_option('lrgawidget_property_id');
			}
			if (version_compare($version, '3.2.0', '<')){
				$results = $wpdb->get_results ( "SELECT `name`, `value` FROM  `".lrgawidget_plugin_table."`", ARRAY_A );
				$settings = array();
				if (!empty($results)){
					foreach ($results as $setting) {
						$settings[$setting['name']] = $setting['value'];
					}			
				}
				if (empty($settings["enable_ecommerce_graph"])){
					$wpdb->insert( lrgawidget_plugin_table, array( 'name' => 'enable_ecommerce_graph', 'value' => 'on'));
				}
			}
			if (!is_multisite()){
				update_option(lrgawidget_plugin_prefiex.'version', lrgawidget_plugin_version, 'yes' );
			}else{
				update_network_option(1, lrgawidget_plugin_prefiex.'version', lrgawidget_plugin_version );
			}
		}
	}

	private static function current_user_id(){
		return get_current_user_id();
	}

	public static function get_current_user_id(){
		return self::$user_id;
	}	

	private static function blog_options(){
		global $wpdb;
		$options = array();
        $results = $wpdb->get_results ( "SELECT `name`, `value` FROM  `".lrgawidget_plugin_table."`", ARRAY_A  );
		if (empty($wpdb->last_error) && !empty($results)){
			foreach ($results as $option) {
				$options[$option['name']] = $option['value'];
			}			
		}
		return $options;
	}
	
	public static function get_blog_option($option){
		$value = "";
		if(!empty(self::$blog_options[$option])){
			$value = sanitize_text_field(self::$blog_options[$option]);
		}
		return $value;
	}
	
	public static function get_current_user_option($option){
		return sanitize_text_field(get_user_option($option, self::$user_id));
	}

	public static function update_current_user_option($option, $value){
		update_user_option(self::$user_id, $option, $value);
	}

	public static function is_analytics($str){
		return (bool) preg_match('/^ua-\d{4,20}(-\d{1,10})?$/i', $str);
	}
	
	public static function sanitizer(&$value) {
	  $value = htmlspecialchars($value, ENT_QUOTES, 'UTF-8');
	}	
	
	public static function get_ga_code(){
		$enable_universal_tracking = self::get_blog_option('enable_universal_tracking');
		if ($enable_universal_tracking === "on"){
			$property_id = self::get_blog_option('property_id');
			if (!current_user_can('edit_posts') && !empty($property_id) && self::is_analytics($property_id)){
?>
<!-- Lara's Google Analytics - https://www.xtraorbit.com/wordpress-google-analytics-dashboard-widget/ -->
<script async src="https://www.googletagmanager.com/gtag/js?id=<?php echo $property_id ?>"></script>
<script>
  window.dataLayer = window.dataLayer || [];
  function gtag(){dataLayer.push(arguments);}
  gtag('js', new Date());
  gtag('config', '<?php echo $property_id ?>', { 'anonymize_ip': true });
</script>
<?php
			}
		}
	}
}
add_action('init', 'Lara\Utils\Common\WPHelperFunctions::initInstance');
?>