<?php

namespace Lara\Utils\Common;

/**
 * @package    Google Analytics by Lara
 * @author     Amr M. Ibrahim <mailamr@gmail.com>
 * @link       https://www.xtraorbit.com/
 * @copyright  Copyright (c) XtraOrbit Web development SRL 2016 - 2020
 */

if (!defined("ABSPATH"))
    die("This file cannot be accessed directly");

class WPReviewNotice {
	
	private static $already_rated;
	private static $install_date;
	private static $install_date_raw;
	private static $current_date;
	private static $current_date_raw;
	private static $difference_in_days;
	
	public static function update_option($option, $value){
		if (!is_multisite()){
			update_option($option, $value, 'yes' );
		}else{
			update_network_option(1, $option, $value );
		}		
	}
	
	public static function show_review_notice(){
        if(current_user_can('update_plugins')){
			self::$already_rated = get_network_option(1,lrgawidget_plugin_prefiex.'already_rated', 'no');
			if (self::$already_rated == 'no'){
				self::update_option(lrgawidget_plugin_prefiex.'already_rated', 'no');
				self::$current_date_raw = date('Y-m-d');
				self::$install_date_raw = get_network_option(1,lrgawidget_plugin_prefiex.'install_date', 'unknown');
				if (self::$install_date_raw != 'unknown'){
					self::$install_date = new \DateTime( self::$install_date_raw );
					self::$current_date = new \DateTime( self::$current_date_raw );
					self::$difference_in_days = (int)self::$install_date->diff(self::$current_date)->format("%r%a");

					if (isset(self::$difference_in_days) && self::$difference_in_days >= 7) {
						add_action('in_admin_header','Lara\Utils\Common\WPReviewNotice::review_notice');
					}

				}else{
					self::update_option(lrgawidget_plugin_prefiex.'install_date', self::$current_date_raw);
				}
			}
			
        }		
	}
	
	public static function review_response(){
		if(current_user_can('update_plugins')){
			if (!empty($_POST["action"]) &&  $_POST["action"] == "lrgawidget_review_response"){
				if ($_POST['rresponse'] === "rated" || $_POST['rresponse'] === "notinterested"){
					self::update_option(lrgawidget_plugin_prefiex.'already_rated', $_POST['rresponse']);					
				}
			}
		}

		header('Content-Type: application/json; charset=utf-8');
		$output['status'] = "done";
		echo json_encode($output);	
		exit();
	}
	
	public static function review_notice(){
		
		$message = __('Thank you for using <b>Lara Google Analytics</b> - WordPress plugin. We hope it has saved your valuable time and efforts!', 'lara-google-analytics');
		$message .= "<br><br>".__('If you have a free moment, and want to help us spread the word and boost our motivation, please do us a BIG favour and give us 5 Stars rating on wordpress .. The more reviews we get, the more cool features we will add to the plugin :)', 'lara-google-analytics');
		
		$btns = '<a href="https://wordpress.org/support/plugin/lara-google-analytics/reviews/" target="_blank"><button class="btn" id="rating" type="button">';
		$btns .= '<span><i aria-hidden="true" class="fa fa-star"></i> <i aria-hidden="true" class="fa fa-star"></i> <i aria-hidden="true" class="fa fa-star"></i> <i aria-hidden="true" class="fa fa-star"></i> <i aria-hidden="true" class="fa fa-star"></i></span>&nbsp;&nbsp;';
		$btns .= __('Let\'s do it, You deserve it', 'lara-google-analytics');
		$btns .= '</button></a>';
		
		$btns .= '<button class="btn " data-lrrateresponse="rated" type="button"><i aria-hidden="true" class="fa fa-thumbs-o-up"></i>&nbsp;&nbsp;';
		$btns .= __('I already rated it', 'lara-google-analytics');
		$btns .= '</button>';
		
		$btns .= '<button class="btn " data-lrrateresponse="notinterested" type="button"><i aria-hidden="true" class="fa fa-ban"></i>&nbsp;&nbsp;';
		$btns .= __('Not Interested', 'lara-google-analytics');
		$btns .= '</button>';		
		

		echo '
		<style>
		#lrgawidget_reviews_notice {
			display: table;
			background-image: url("'.lrgawidget_plugin_dist_dir.'/img/xo_footer.png");
			background-position: center bottom;
			background-repeat: repeat-x;
			background-color: #f9f9f9;
			height: 150px;
			padding: 10px;
		}
		
		#lrgawidget_reviews_notice .lrxologo {
			display: table-cell;
			background-image: url("'.lrgawidget_plugin_dist_dir.'/img/xtraorbit_logo_small.png");
			background-repeat: no-repeat;
			background-position: center top;
			width: 400px;
			height: 150px;
			margin-top: 15px;
		}
		
		#lrgawidget_reviews_notice .lrnoticelogo {
			display: table-cell;
			vertical-align: middle;
		}	

		#lrgawidget_reviews_notice .lrnoticemessage {
			display: table-cell;
			border: 25px solid transparent;
			vertical-align: top;
		}

		#lrgawidget_reviews_notice .btn {
			background-color: #272e38;
			color: #FFFFFF;
			border: 1px solid transparent;
			cursor: pointer;
			margin: 0px 10px 10px 0px;
			min-height: 40px;
			border-radius: 5px;
		}
		
		#lrgawidget_reviews_notice .btn:hover {
			background-color: #f7981d;
		}		
		
		</style>
		
		<script type="text/javascript">
		(function($) {
			function lrWidgetSettings(arr){
				return $.ajax({
					method: "POST",
					url: lrgawidget_ajax_object.lrgawidget_ajax_url,
					data: arr,
					dataType: "json"
				});
			}
			
			$(document).ready(function(){
				$("[data-lrrateresponse]").on("click", function (e) {
					lrWidgetSettings({action : "lrgawidget_review_response", rresponse: $(this).data("lrrateresponse")}).done(function (data, textStatus, jqXHR) {
						$("#lrgawidget_reviews_notice").slideUp("fast");
					});					
				});
			});
		})(jQuery);
		</script>


		<div id="lrgawidget_reviews_notice" class="notice notice-info is-dismissible">
					<div class="lrnoticelogo"><img src="'.lrgawidget_plugin_dist_dir.'/img/lrwidget_logo.png"></div>
					<div class="lrnoticemessage">'.$message.'<br><br>'.$btns.'</div>
					<a href="https://www.xtraorbit.com/wordpress-google-analytics-dashboard-widget/?utm_source=InApp&amp;utm_medium=Rate_Screen" target="_blank"><div class="lrxologo">&nbsp;</div></a>
					
					<button type="button" class="notice-dismiss">
						<span class="screen-reader-text">Dismiss this notice.</span>
					</button>
				</div>';
	}	
}

?>